﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Text;
using System.Linq;

namespace Cp.Core
{
   public class CaseContainer
    {
        static Dictionary<string, ITest> dictTests = new Dictionary<string, ITest>();

        static void InitializeMenu()
        {
            var tests = Assembly
                .GetEntryAssembly()
               .GetTypes()
               .Where(t => typeof(ITest).IsAssignableFrom(t) && t != typeof(ITest));

            int index = 1;
            foreach (Type test in tests)
            {
                dictTests.Add(index.ToString(), (ITest)Activator.CreateInstance(test));
                index++;
            }
        }

        static void ShowMenu()
        {
            Console.Clear();
            Console.WriteLine("All Tests in the project:");
            Console.WriteLine(new string('*', 60));
            foreach (var dictKeyValue in dictTests)
            {
                Console.WriteLine(string.Format("{0}. {1}", dictKeyValue.Key, dictKeyValue.Value.Name));
            }
             Console.WriteLine(new string('*',60));
            Console.WriteLine("Press 'Enter' To Run all samples one by one. Press 'Esc' to exist");
        }

        static void Run()
        {
            ShowMenu();
            var key = Console.ReadKey();
            if (key.Key == ConsoleKey.Escape)
            {
                return;
            } 
            var input =key.KeyChar+ Console.ReadLine();
            if (dictTests.ContainsKey(input))
            {
                Console.Clear();
                Console.WriteLine('['+dictTests[input].Name+']');
                Console.WriteLine(dictTests[input].Description);
                Console.WriteLine(new string('*', 60));
                dictTests[input].Test();
                Console.WriteLine(new string('*', 60));
                Console.WriteLine("Press 'Esc' to quit program. Press any other key to go back to menu.");
            }
            else
            {
                Console.WriteLine("Please enter an invalid index!");
            }
            if (Console.ReadKey().Key != ConsoleKey.Escape)
            {
                Run();
            }
        }

        public static void ShowCases()
        {
            //如果要显示中文，取消注释这个。
            //Console.OutputEncoding = Encoding.GetEncoding(936);
            InitializeMenu();
            Run();
        }
    }
}
