﻿using Cp.Core;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;
using System.Reflection.Emit;
using System.Text;

namespace Cp.EmitStudy
{
    //http://www.codeproject.com/Articles/121568/Dynamic-Type-Using-Reflection-Emit#heading0017
    class Test1:ITest
    {
        public string Name
        {
            get
            {
                return "Type Builder Test";
            }
        }

        public string Description
        {
            get
            {
                return "Type Builder Test";
            }
        }

        public void Test()
        {
            var name = "Test1";
            
            var assembly = AppDomain.CurrentDomain.DefineDynamicAssembly(new AssemblyName(name), AssemblyBuilderAccess.RunAndCollect);

            var module = assembly.DefineDynamicModule(name);
            var typeBuilder = module.DefineType(name, TypeAttributes.Public, typeof(object));

            var Namefield = typeBuilder.DefineField("name", typeof(string), FieldAttributes.Private);
            var propertyName = typeBuilder.DefineProperty("Name", PropertyAttributes.HasDefault, typeof(string), null);
            var getNameMethod = typeBuilder.DefineMethod("get_Name", MethodAttributes.Public |
    MethodAttributes.SpecialName |
    MethodAttributes.HideBySig, typeof(string), Type.EmptyTypes);
            var il = getNameMethod.GetILGenerator();
            il.Emit(OpCodes.Ldarg_0);
            il.Emit(OpCodes.Ldfld, Namefield);
            il.Emit(OpCodes.Ret);
            propertyName.SetGetMethod(getNameMethod);

            var setNameMethod = typeBuilder.DefineMethod("set_Name", MethodAttributes.Public |
    MethodAttributes.SpecialName |
    MethodAttributes.HideBySig, null, new Type[]{typeof(string)});
            il = setNameMethod.GetILGenerator();
            il.Emit(OpCodes.Ldarg_0);
            il.Emit(OpCodes.Ldarg_1);
            il.Emit(OpCodes.Stfld, Namefield);
            il.Emit(OpCodes.Ret);
            propertyName.SetSetMethod(setNameMethod);

            var ctroAttributes = MethodAttributes.Public |
MethodAttributes.HideBySig | MethodAttributes.SpecialName | MethodAttributes.RTSpecialName;
            var ctroBuilder0 = typeBuilder.DefineConstructor(ctroAttributes,CallingConventions.Standard, new Type[0] );
            il = ctroBuilder0.GetILGenerator();
            il.Emit(OpCodes.Ldarg_0);
            il.Emit(OpCodes.Call, typeof(object).GetConstructor(Type.EmptyTypes));
            il.Emit(OpCodes.Ret);

            var ctroBuilder1 = typeBuilder.DefineConstructor(ctroAttributes, CallingConventions.Standard,new Type[]{typeof(string)});
            il = ctroBuilder1.GetILGenerator();
            il.Emit(OpCodes.Ldarg_0);
            il.Emit(OpCodes.Call, typeof(object).GetConstructor(Type.EmptyTypes));
            il.Emit(OpCodes.Ldarg_0);
            il.Emit(OpCodes.Ldarg_1);
            il.Emit(OpCodes.Call, setNameMethod);
            il.Emit(OpCodes.Ret);

            var divideMethodAttribute = MethodAttributes.Public ;
            var divideMethod = typeBuilder.DefineMethod("Divide", divideMethodAttribute,    CallingConventions.Standard, typeof(System.Single), new Type[] { typeof(System.Int32), typeof(System.Int32) });
            divideMethod.SetImplementationFlags(MethodImplAttributes.Managed);
            il= divideMethod.GetILGenerator();
            il.DeclareLocal(typeof(DivideByZeroException));
            il.DeclareLocal(typeof(float));
            il.Emit(OpCodes.Nop);
            Label lblTry = il.BeginExceptionBlock();
            il.Emit(OpCodes.Ldarg_1);
            il.Emit(OpCodes.Ldarg_2);
            il.Emit(OpCodes.Div);
            il.Emit(OpCodes.Conv_R4); // Converts to Float32
            il.Emit(OpCodes.Stloc_1);
            il.Emit(OpCodes.Leave_S, lblTry);
            il.BeginCatchBlock(typeof(DivideByZeroException));
            il.Emit(OpCodes.Stloc_0);
            il.Emit(OpCodes.Ldstr, "ZeroDivide exception : {0}");
            il.Emit(OpCodes.Ldloc_0);
            MethodInfo minfo = typeof(DivideByZeroException).GetMethod("get_Message");
            il.Emit(OpCodes.Callvirt, minfo);
            MethodInfo wl = typeof(Console).GetMethod("WriteLine", new Type[] { typeof(string), typeof(object) });
            il.Emit(OpCodes.Call, wl);
            il.Emit(OpCodes.Ldc_R4, 0.0);
            il.Emit(OpCodes.Stloc_1);
            il.Emit(OpCodes.Leave_S, lblTry);
            il.EndExceptionBlock();
            il.Emit(OpCodes.Ldloc_1);
            il.Emit(OpCodes.Ret);

            var type=typeBuilder.CreateType();

            var expNew = Expression.New(type);
            var lambdaNew = Expression.Lambda<Func<object>>(expNew);
            var funcNew = lambdaNew.Compile();
            var objNew = funcNew();

            var propName=type.GetProperty("Name");
            Console.WriteLine("PropName Reflected Type: " + propName.ReflectedType);
            propName.SetValue(objNew,"Name Property Set By Reflection!",null);
            Console.WriteLine("Name Value is: "+propName.GetValue(objNew,null));

            objNew = Activator.CreateInstance(type, "Create Object By Constructor");
            Console.WriteLine("Name Value is: " + propName.GetValue(objNew,null));

            var value = type.GetMethod("Divide").Invoke(objNew, new object[] { 10, 5 });
            Console.WriteLine("Call Divide Method(10/5): "+value);
        }


    }

   
}
/*About Ldarg_0
 * Instance methods have an implicit parameter called "this". It is loaded as the first argument to the stack, thus you have ldarg.0 for "this".
* http://stackoverflow.com/questions/1785372/why-do-i-have-to-do-ldarg-0-before-calling-a-field-in-msil 
 
 */